﻿using AspNetCoreLecture.Data.Repository;
using AspNetCoreLecture.Models;
using Microsoft.AspNetCore.Mvc;

namespace AspNetCoreLecture.Controllers
{
    public class GradeController : Controller
    {
        private readonly IGradeRepository _gradeRepository;
        private readonly IStudentRepository _studentRepository;
        private readonly ICourseRepository _courseRepository;

        public GradeController(IGradeRepository gradeRepository, IStudentRepository studentRepository, ICourseRepository courseRepository)
        {
            _gradeRepository = gradeRepository;
            _studentRepository = studentRepository;
            _courseRepository = courseRepository;
        }

        public IActionResult Index(int studentId)
        {
            var studentDto = _studentRepository.Get(studentId);
            if (studentDto == null)
            {
                return NotFound();
            }

            var grades = _gradeRepository.GetAllForStudent(studentId);

            var courses = _courseRepository.Get(grades.Select(x => x.CourseId).Distinct());

            var model = new GradeIndexViewModel()
            {
                Student = studentDto,
                Grades = grades.Join(courses, x => x.CourseId, x => x.Id, (g, c) => new { Grade = g, Course = c }).ToDictionary(x => x.Course, x => x.Grade)
            };

            return View(model);
        }

        public IActionResult Add(int studentId)
        {
            var studentDto = _studentRepository.Get(studentId);
            if (studentDto is null)
            {
                return NotFound();
            }

            var gradedCourseIds = _gradeRepository.GetAllForStudent(studentId).Select(x => x.CourseId).Distinct();

            var model = new GradeInsertViewModel()
            {
                StudentId = studentId,
                StudentName = $"{studentDto.FirstName} {studentDto.LastName}",
                AvailableCourses = _courseRepository.GetAll().Where(x => !gradedCourseIds.Contains(x.Id)).ToArray()
            };

            return View(model);
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public IActionResult Add(GradeInsertViewModel model)
        {
            if (!ModelState.IsValid)
            {
                var studentDto = _studentRepository.Get(model.StudentId);
                model.StudentName = $"{studentDto.FirstName} {studentDto.LastName}";
                var gradedCourseIds = _gradeRepository.GetAllForStudent(model.StudentId).Select(x => x.CourseId).Distinct();
                model.AvailableCourses = _courseRepository.GetAll().Where(x => !gradedCourseIds.Contains(x.Id)).ToArray();
                return View(model);
            }

            _gradeRepository.Create(model.ToDto());

            return RedirectToAction(nameof(Index), new { studentId = model.StudentId });
        }

        public IActionResult Edit(int gradeId)
        {
            var gradeDto = _gradeRepository.Get(gradeId);
            if (gradeDto is null)
            {
                return NotFound();
            }

            var courseDto = _courseRepository.Get(gradeDto.CourseId);
            var studentDto = _studentRepository.Get(gradeDto.StudentId);

            var model = new GradeUpdateViewModel()
            {
                CourseId = gradeDto.CourseId,
                CourseName = courseDto.Name,
                Grade = gradeDto.GradeLetter,
                Id = gradeId,
                StudentId = gradeDto.StudentId,
                StudentName = $"{studentDto.FirstName} {studentDto.LastName}"
            };

            return View(model);
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public IActionResult Edit(GradeUpdateViewModel model)
        {
            if (!ModelState.IsValid)
            {
                var courseDto = _courseRepository.Get(model.CourseId);
                var studentDto = _studentRepository.Get(model.StudentId);
                model.CourseName = courseDto.Name;
                model.StudentName = $"{studentDto.FirstName} {studentDto.LastName}";
                return View(model);
            }

            _gradeRepository.Update(model.ToDto());

            return RedirectToAction(nameof(Index), new { studentId = model.StudentId });
        }
    }
}
